--
-- ExtendedDynamicMountAttacher
-- Class for baleforks which got more than one trigger (max 4 triggers, is there a way to dynamically add triggers?) 
--
-- @author  Stefan Maurus	
-- @date  2015-05-10

-- Copyright © Stefan Maurus, www.stefanmaurus.de


ExtendedDynamicMountAttacher = {};

function ExtendedDynamicMountAttacher.prerequisitesPresent(specializations)
    return true;
end;

function ExtendedDynamicMountAttacher:load(xmlFile)
	self.dynamicMountAllTriggers = SpecializationUtil.callSpecializationsFunction("dynamicMountAllTriggers");

	self.dynamicMountTrigger1CallbackEnter = SpecializationUtil.callSpecializationsFunction("dynamicMountTrigger1CallbackEnter");
	self.dynamicMountTrigger2CallbackEnter = SpecializationUtil.callSpecializationsFunction("dynamicMountTrigger2CallbackEnter");
	self.dynamicMountTrigger3CallbackEnter = SpecializationUtil.callSpecializationsFunction("dynamicMountTrigger3CallbackEnter");
	self.dynamicMountTrigger4CallbackEnter = SpecializationUtil.callSpecializationsFunction("dynamicMountTrigger4CallbackEnter");
	
	self.dynamicMountTrigger1CallbackLeave = SpecializationUtil.callSpecializationsFunction("dynamicMountTrigger1CallbackLeave");
	self.dynamicMountTrigger2CallbackLeave = SpecializationUtil.callSpecializationsFunction("dynamicMountTrigger2CallbackLeave");
	self.dynamicMountTrigger3CallbackLeave = SpecializationUtil.callSpecializationsFunction("dynamicMountTrigger3CallbackLeave");
	self.dynamicMountTrigger4CallbackLeave = SpecializationUtil.callSpecializationsFunction("dynamicMountTrigger4CallbackLeave");	

	self.addDynamicMountedObject = SpecializationUtil.callSpecializationsFunction("addDynamicMountedObject");
	self.removeDynamicMountedObject = SpecializationUtil.callSpecializationsFunction("removeDynamicMountedObject");
	
	self.removeFromPending = SpecializationUtil.callSpecializationsFunction("removeFromPending");
	
	self.mountDynamicWM = ExtendedDynamicMountAttacher.mountDynamicWM;
	
	self.groundContactForce = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.dynamicMountAttacher#force"), 2);
	
	self.dynamicMountTriggers = {};
	local i = 0;
	while true do
		local key = string.format("vehicle.dynamicMountAttacher.trigger(%d)", i);
		if not hasXMLProperty(xmlFile, key) then
			break;
		end;
		if self.isServer then
			local attacherTriggerTriggerNode = Utils.indexToObject(self.components, getXMLString(xmlFile, key.."#enterTrigger"));
			local leaveTriggerTriggerNode = Utils.indexToObject(self.components, getXMLString(xmlFile, key.."#leaveTrigger"));			
			local attacherTriggerRootNode = Utils.indexToObject(self.components, getXMLString(xmlFile, key.."#rootNode"));
			local attacherTriggerJointNode = Utils.indexToObject(self.components, getXMLString(xmlFile, key.."#jointNode"));
			
			local timeToAttach = Utils.getNoNil(getXMLFloat(xmlFile, key.."#timeToAttach"), 500);
			addTrigger(attacherTriggerTriggerNode, string.format("dynamicMountTrigger%dCallbackEnter", i+1), self);
			addTrigger(leaveTriggerTriggerNode, string.format("dynamicMountTrigger%dCallbackLeave", i+1), self);
			
			local dynamicMountAttacherTrigger = {triggerNode=attacherTriggerTriggerNode, rootNode=attacherTriggerRootNode, jointNode=attacherTriggerJointNode, leaveTrigger=leaveTriggerTriggerNode};

			table.insert(self.dynamicMountTriggers, {dynamicMountAttacherTrigger=dynamicMountAttacherTrigger, timeToAttach=timeToAttach});
		end;
		i=i+1;
	end;
	
	self.dynamicMountedObjects={};
	self.dynamicMountedTypes={};	
	self.dynamicMountedTriggers={};

	self.pendingDynamicMountObjects={};
	self.pendingDynamicMountTriggers={};	
	self.pendingDynamicMountTimes = {};
	
	self.debugMode = false;
end;

function ExtendedDynamicMountAttacher:delete()
    if self.isServer then
		for _,object in pairs(self.dynamicMountedObjects) do
			object:unmountDynamic();
		end
    end
	
	for _,trigger in pairs(self.dynamicMountTriggers) do
		removeTrigger(trigger.dynamicMountAttacherTrigger.triggerNode);
		removeTrigger(trigger.dynamicMountAttacherTrigger.leaveTrigger);
	end		
end;

function ExtendedDynamicMountAttacher:mouseEvent(posX, posY, isDown, isUp, button)
end;

function ExtendedDynamicMountAttacher:keyEvent(unicode, sym, modifier, isDown)
end;

function ExtendedDynamicMountAttacher:update(dt)
end;

function ExtendedDynamicMountAttacher:updateTick(dt)
    if self.isServer then
		for v,object in ipairs(self.pendingDynamicMountObjects) do
			if self.pendingDynamicMountTimes[v] ~= nil then
				self.pendingDynamicMountTimes[v] = self.pendingDynamicMountTimes[v] + dt;
			end;

			if not Utils.hasListElement(self.dynamicMountedObjects, object) then -- is not already mounted
				local trigger = self.dynamicMountTriggers[self.pendingDynamicMountTriggers[v]];
				
				if trigger.timeToAttach < self.pendingDynamicMountTimes[v] then
					if object:mountDynamic(self, trigger.dynamicMountAttacherTrigger.rootNode, trigger.dynamicMountAttacherTrigger.jointNode, DynamicMountUtil.TYPE_FORK, 10) then
						self:addDynamicMountedObject(object);
						table.insert(self.dynamicMountedTriggers, Utils.findListElementFirstIndex(self.dynamicMountTriggers, trigger, nil));
						table.insert(self.dynamicMountedTypes, DynamicMountUtil.TYPE_FORK);
					end;
				end
			end
		end
		
		for v,object in ipairs(self.dynamicMountedObjects) do
			local trigger = self.dynamicMountTriggers[self.dynamicMountedTriggers[v]];
			
			local groundRaycastResult = {
				raycastCallback = function (self, transformId, x, y, z, distance)
					self.groundDistance = distance;
				end
			};
			
			local offset = Utils.getNoNil(object.baleDiameter, 0.8);

			local x,y,z = getWorldTranslation(object.nodeId);
			groundRaycastResult.groundDistance = 0;
			raycastClosest(x, y-offset/2, z, 0, -1, 0, "raycastCallback", 4, groundRaycastResult, 476);
			
			if self.debugMode then
				drawDebugLine(x, y, z,1,0,0,x,0,z,1,0,0);
				drawDebugPoint(x, y-offset/2, z, 0,1,0,1);
			end;
			
			if groundRaycastResult.groundDistance > 0.2 then
				if self.dynamicMountedTypes[v] == DynamicMountUtil.TYPE_FORK then
					removeJoint(object.dynamicMountJointIndex);
					object.dynamicMountJointIndex = nil;
					object.dynamicMountObjectActorId = nil;
					object.dynamicMountObject = nil;	

					local b = self:mountDynamicWM(object, trigger.dynamicMountAttacherTrigger.rootNode, trigger.dynamicMountAttacherTrigger.jointNode);

					self.dynamicMountedTypes[v] = DynamicMountUtil.TYPE_AUTO_ATTACH_XYZ;
				end;
			else
				if self.dynamicMountedTypes[v] == DynamicMountUtil.TYPE_AUTO_ATTACH_XYZ then
					removeJoint(object.dynamicMountJointIndex);
					object.dynamicMountJointIndex = nil;
					object.dynamicMountObjectActorId = nil;
					object.dynamicMountObject = nil;	
					
					local b = object:mountDynamic(self, trigger.dynamicMountAttacherTrigger.rootNode, trigger.dynamicMountAttacherTrigger.jointNode, DynamicMountUtil.TYPE_FORK, self.groundContactForce);
					
					self.dynamicMountedTypes[v] = DynamicMountUtil.TYPE_FORK;
				end;
			end;
		end;
    end
end;

function ExtendedDynamicMountAttacher:addDynamicMountedObject(object)
	if not Utils.hasListElement(self.dynamicMountedObjects, object) then
		table.insert(self.dynamicMountedObjects, object);
	end;
	
	local index = Utils.findListElementFirstIndex(self.pendingDynamicMountObjects, object, nil);
	if index ~= nil then
		self:removeFromPending(index);
	end;	
end

function ExtendedDynamicMountAttacher:removeDynamicMountedObject(object, isDeleting)
	local index = Utils.findListElementFirstIndex(self.dynamicMountedObjects, object, nil);
	if index ~= nil then
		table.remove(self.dynamicMountedObjects, index);
		table.remove(self.dynamicMountedTriggers, index);	
		table.remove(self.dynamicMountedTypes, index);			
	end;

    if isDeleting then
		local index = Utils.findListElementFirstIndex(self.pendingDynamicMountObjects, object, nil);
		if index ~= nil then
			self:removeFromPending(index);
		end;
    end
end

function ExtendedDynamicMountAttacher:draw()
	if self.debugMode then
		for i,object in ipairs(self.dynamicMountedTypes) do
			renderText(0.5, 0.05*i, 0.02, i.." "..tostring(object));
		end;

		for i,object in ipairs(self.dynamicMountedTriggers) do
			renderText(0.4, 0.05*i, 0.02, i.." "..tostring(object));
		end;

		for i,object in ipairs(self.dynamicMountedObjects) do
			renderText(0.3, 0.05*i, 0.02, i.." "..tostring(object));
		end;
		
		for i,object in ipairs(self.pendingDynamicMountObjects) do
			renderText(0.2, 0.05*i, 0.02, i.." "..tostring(object));
		end;
		
		for i,object in ipairs(self.pendingDynamicMountTriggers) do
			renderText(0.1, 0.05*i, 0.02, tostring(object));
		end;
		
		for i,v in ipairs(self.pendingDynamicMountTimes) do
			renderText(0.02, 0.05*i, 0.02, v);
		end;
	end;
end;

function ExtendedDynamicMountAttacher:dynamicMountAllTriggers(triggerId, otherActorId, onEnter, onLeave, onStay, otherShapeId, i)
    if onEnter then
        local object = Utils.getNoNil(g_currentMission:getNodeObject(otherActorId), g_currentMission.nodeToVehicle[otherActorId]); --get object
		
        if object ~= nil and object.getSupportsMountDynamic ~= nil and object:getSupportsMountDynamic() and object.lastMoveTime ~= nil then -- is bale?
			if not Utils.hasListElement(self.pendingDynamicMountObjects, object) and not Utils.hasListElement(self.dynamicMountedObjects, object) then -- not already pending
				table.insert(self.pendingDynamicMountObjects, object); -- add object to pending objects
				table.insert(self.pendingDynamicMountTriggers, i); -- add trigger index
				table.insert(self.pendingDynamicMountTimes, 0); -- reset trigger counter
			end;
        end
    elseif onLeave then
        local object = Utils.getNoNil(g_currentMission:getNodeObject(otherActorId), g_currentMission.nodeToVehicle[otherActorId]); --get object
		
        if object ~= nil then
			for c,objectPending in ipairs(self.pendingDynamicMountObjects) do
				if objectPending == object then
					self:removeFromPending(c);
				end;
			end
			
			local index = Utils.findListElementFirstIndex(self.dynamicMountedObjects, object, nil);
			if index ~= nil and self.dynamicMountedTriggers[index] == i then -- remove it it is mounted and leaves the right trigger
				self:removeDynamicMountedObject(object, false); 
				object:unmountDynamic();
			end;
        end
    end
end;

--enter trigger only calls if a object enters

function ExtendedDynamicMountAttacher:dynamicMountTrigger1CallbackEnter(triggerId, otherActorId, onEnter, onLeave, onStay, otherShapeId)
    self:dynamicMountAllTriggers(triggerId, otherActorId, onEnter, false, false, otherShapeId, 1);
end;

function ExtendedDynamicMountAttacher:dynamicMountTrigger2CallbackEnter(triggerId, otherActorId, onEnter, onLeave, onStay, otherShapeId)
    self:dynamicMountAllTriggers(triggerId, otherActorId, onEnter, false, false, otherShapeId, 2);
end;

function ExtendedDynamicMountAttacher:dynamicMountTrigger3CallbackEnter(triggerId, otherActorId, onEnter, onLeave, onStay, otherShapeId)
    self:dynamicMountAllTriggers(triggerId, otherActorId, onEnter, false, false, otherShapeId, 3);
end;

function ExtendedDynamicMountAttacher:dynamicMountTrigger4CallbackEnter(triggerId, otherActorId, onEnter, onLeave, onStay, otherShapeId)
    self:dynamicMountAllTriggers(triggerId, otherActorId, onEnter, false, false, otherShapeId, 4);
end;

--leave triggers only call if a object leaves

function ExtendedDynamicMountAttacher:dynamicMountTrigger1CallbackLeave(triggerId, otherActorId, onEnter, onLeave, onStay, otherShapeId)
    self:dynamicMountAllTriggers(triggerId, otherActorId, false, onLeave, false, otherShapeId, 1);
end;

function ExtendedDynamicMountAttacher:dynamicMountTrigger2CallbackLeave(triggerId, otherActorId, onEnter, onLeave, onStay, otherShapeId)
    self:dynamicMountAllTriggers(triggerId, otherActorId, false, onLeave, false, otherShapeId, 2);
end;

function ExtendedDynamicMountAttacher:dynamicMountTrigger3CallbackLeave(triggerId, otherActorId, onEnter, onLeave, onStay, otherShapeId)
    self:dynamicMountAllTriggers(triggerId, otherActorId, false, onLeave, false, otherShapeId, 3);
end;

function ExtendedDynamicMountAttacher:dynamicMountTrigger4CallbackLeave(triggerId, otherActorId, onEnter, onLeave, onStay, otherShapeId)
    self:dynamicMountAllTriggers(triggerId, otherActorId, false, onLeave, false, otherShapeId, 4);
end;

function ExtendedDynamicMountAttacher:removeFromPending(index)
	table.remove(self.pendingDynamicMountObjects, index); --remove pending object if it leaves trigger
	table.remove(self.pendingDynamicMountTriggers, index); --remove trigger index if it leaves trigger
	table.remove(self.pendingDynamicMountTimes, index);  -- remove counter
end;

function ExtendedDynamicMountAttacher:mountDynamicWM(object, objectActorId, jointNode)
    if not object:getSupportsMountDynamic() or object.mountObject ~= nil or object.dynamicMountObject ~= nil then
        return false;
    end
	
    local constr = JointConstructor:new();
    constr:setActors(objectActorId, object.nodeId);
    constr:setJointTransforms(jointNode, jointNode);

    constr:setRotationLimit(0, 0, 0);
    constr:setRotationLimit(1, 0, 0);
    constr:setRotationLimit(2, 0, 0);

	constr:setTranslationLimit(0, true, -0, 0);
	constr:setTranslationLimit(1, true, -0, 0);
	constr:setTranslationLimit(2, true, -0, 0);
	constr:setEnableCollision(true);

    object.dynamicMountJointIndex = constr:finalize();
    object.dynamicMountObjectActorId = objectActorId;
    object.dynamicMountObject = self;
	
    return true;
end