--
-- MurrayOctaQuad_back
-- Specialization for Murray Octa-Quad back animations and overlay
--
-- @author  Stefan Maurus	
-- @date  2015-03-24

-- Copyright © Stefan Maurus, www.stefanmaurus.de

MurrayOctaQuad_back = {};
MurrayOctaQuad_back.modDir = g_currentModDirectory;

function MurrayOctaQuad_back.prerequisitesPresent(specializations)
    return true;
end;

function MurrayOctaQuad_back:load(xmlFile)
	self.open = SpecializationUtil.callSpecializationsFunction("open");
	self.close = SpecializationUtil.callSpecializationsFunction("close");

	self.f_foldRightFront = SpecializationUtil.callSpecializationsFunction("f_foldRightFront");
	self.f_rightFront = SpecializationUtil.callSpecializationsFunction("f_rightFront");
	self.f_foldRightBack = SpecializationUtil.callSpecializationsFunction("f_foldRightBack");
	self.f_rightBack = SpecializationUtil.callSpecializationsFunction("f_rightBack");
	
	self.f_foldLeftFront = SpecializationUtil.callSpecializationsFunction("f_foldLeftFront");
	self.f_leftFront = SpecializationUtil.callSpecializationsFunction("f_leftFront");
	self.f_foldLeftBack = SpecializationUtil.callSpecializationsFunction("f_foldLeftBack");	
	self.f_leftBack = SpecializationUtil.callSpecializationsFunction("f_leftBack");

	self.putGlobalTogether = SpecializationUtil.callSpecializationsFunction("putGlobalTogether");
	self.putBackTogether = SpecializationUtil.callSpecializationsFunction("putBackTogether");
	self.putFrontTogether = SpecializationUtil.callSpecializationsFunction("putFrontTogether");

	self.setAnimationState = SpecializationUtil.callSpecializationsFunction("setAnimationState");
	
	local i = 0;
	self.foldAnimations = {};
	while true do
		local key = string.format("vehicle.foldAnimations.animation(%d)", i);
		if not hasXMLProperty(xmlFile, key) then
			break;
		end;
		
		local name = getXMLString(xmlFile, key.."#name");	
		local speed = getXMLFloat(xmlFile, key.."#speed");
		
		table.insert(self.foldAnimations, {name=name, speed=speed, state=false});
		i = i + 1;
	end;
	
	local size = 512;
	
	self.currentSelection = 0;
	
	self.octaControl = {};
	local width, height = getNormalizedScreenValues(size, size);
	self.octaControl.xPos = 1-width;
	self.octaControl.yPos = 0.5-height/2;

	self.gui = RafftnixGUI:new(nil, false, true);
	
	self.octaControl.backgroundImage = self.gui:createImage(self.gui.baseElement, self.octaControl.xPos, self.octaControl.yPos, width, height, "textures/controls.dds");
	self.octaControl.headerText = self.gui:createText(self.octaControl.backgroundImage, 0, 0, width, height+height*0.01, g_i18n:getText("OCTACONTROL_HEADER"), 0.02, {1,1,1,1});
	self.octaControl.headerText:setTextFocus("middle","top");
	
	self.octaControl.actionBackground = self.gui:createImage(self.octaControl.backgroundImage, 0, 0, width*0.7, height*0.045, "textures/background.dds");
	
	self.octaControl.currentAction = self.gui:createText(self.octaControl.backgroundImage, width*0.01, height*0.01, width, height, "", 0.015, {1,1,1,1});
	self.octaControl.currentAction:setTextFocus("left","bottom");
	
	local crossX, crossY = getNormalizedScreenValues(25, 25);
	
	self.octaControl.closeBottun = self.gui:createImgButton(self.octaControl.backgroundImage, width-crossX*2, height, crossX, crossY, "textures/button_cross.dds", self.close, self);
	self.octaControl.closeBottun:registerMouseOver("textures/button_cross.dds");	
	
	local _, buttonLayoutOffset = getNormalizedScreenValues(0, size/512*37);
	
	self.octaControl.buttonLayout = self.gui:createElement(self.octaControl.backgroundImage, 0, buttonLayoutOffset, width, height);
	
	local buttonX, buttonY = getNormalizedScreenValues(size/512*100, size/512*100);
	
	local width, height = getNormalizedScreenValues(size/512*47, size/512*15);
	self.octaControl.foldRightFront = self.gui:createImgButton(self.octaControl.buttonLayout, width, height, buttonX, buttonY, "textures/button_foldTines.dds", self.f_foldRightFront, self);
	self.octaControl.foldRightFront:registerMouseOver("textures/button_mouseover.dds");
	
	local width, height = getNormalizedScreenValues(size/512*47, size/512*123);
	self.octaControl.rightFront = self.gui:createImgButton(self.octaControl.buttonLayout, width, height, buttonX, buttonY, "textures/button_foldArm.dds", self.f_rightFront, self);
	self.octaControl.rightFront:registerMouseOver("textures/button_mouseover.dds");
	
	local width, height = getNormalizedScreenValues(size/512*47, size/512*225);
	self.octaControl.foldRightBack = self.gui:createImgButton(self.octaControl.buttonLayout, width, height, buttonX, buttonY, "textures/button_foldTines.dds", self.f_foldRightBack, self);
	self.octaControl.foldRightBack:registerMouseOver("textures/button_mouseover.dds");
	
	local width, height = getNormalizedScreenValues(size/512*47, size/512*331);
	self.octaControl.rightBack = self.gui:createImgButton(self.octaControl.buttonLayout, width, height, buttonX, buttonY, "textures/button_foldArm.dds", self.f_rightBack, self);
	self.octaControl.rightBack:registerMouseOver("textures/button_mouseover.dds");
	
	
	local width, height = getNormalizedScreenValues(size/512*360, size/512*15);
	self.octaControl.foldLeftFront = self.gui:createImgButton(self.octaControl.buttonLayout, width, height, buttonX, buttonY, "textures/button_foldTines.dds", self.f_foldLeftFront, self);
	self.octaControl.foldLeftFront:registerMouseOver("textures/button_mouseover.dds");
	
	local width, height = getNormalizedScreenValues(size/512*360, size/512*123);
	self.octaControl.leftFront = self.gui:createImgButton(self.octaControl.buttonLayout, width, height, buttonX, buttonY, "textures/button_foldArm.dds", self.f_leftFront, self);
	self.octaControl.leftFront:registerMouseOver("textures/button_mouseover.dds");
	
	local width, height = getNormalizedScreenValues(size/512*360, size/512*225);
	self.octaControl.foldLeftBack = self.gui:createImgButton(self.octaControl.buttonLayout, width, height, buttonX, buttonY, "textures/button_foldTines.dds", self.f_foldLeftBack, self);
	self.octaControl.foldLeftBack:registerMouseOver("textures/button_mouseover.dds");
	
	local width, height = getNormalizedScreenValues(size/512*360, size/512*331);
	self.octaControl.leftBack = self.gui:createImgButton(self.octaControl.buttonLayout, width, height, buttonX, buttonY, "textures/button_foldArm.dds", self.f_leftBack, self);
	self.octaControl.leftBack:registerMouseOver("textures/button_mouseover.dds");
	
	local openLockFilename = "textures/button_openLock.dds";
	local closedLockFilename = "textures/button_closedLock.dds";
	
	local buttonX, buttonY = getNormalizedScreenValues(size/512*186, size/512*36);
	
	local width, height = getNormalizedScreenValues(size/512*166, size/512*216);
	self.octaControl.globalTogether = self.gui:createImgButton(self.octaControl.buttonLayout, width, height, buttonX, buttonY, openLockFilename, self.putGlobalTogether, self);
	self.octaControl.globalTogether:registerMouseOver("textures/button_mouseover_lock.dds");	
	
	local width, height = getNormalizedScreenValues(size/512*166, size/512*318);
	self.octaControl.backTogether = self.gui:createImgButton(self.octaControl.buttonLayout, width, height, buttonX, buttonY, openLockFilename, self.putBackTogether, self);
	self.octaControl.backTogether:registerMouseOver("textures/button_mouseover_lock.dds");
	
	local width, height = getNormalizedScreenValues(size/512*166, size/512*99);
	self.octaControl.frontTogether = self.gui:createImgButton(self.octaControl.buttonLayout, width, height, buttonX, buttonY, openLockFilename, self.putFrontTogether, self);
	self.octaControl.frontTogether:registerMouseOver("textures/button_mouseover_lock.dds");
	
	self.openLockOverlay = createImageOverlay(MurrayOctaQuad_back.modDir..openLockFilename);
	self.closedLockOverlay = createImageOverlay(MurrayOctaQuad_back.modDir..closedLockFilename);

	self.globalTogether = false;
	self.backTogether = false;
	self.frontTogether = false;
end;

function MurrayOctaQuad_back:delete()
end;

function MurrayOctaQuad_back:mouseEvent(posX, posY, isDown, isUp, button)
end;

function MurrayOctaQuad_back:keyEvent(unicode, sym, modifier, isDown)
end;

function MurrayOctaQuad_back:readStream(streamId, connection)
	for i = 1, 8 do
		self:setAnimationState(i, streamReadBool(streamId), true);
	end;
end;

function MurrayOctaQuad_back:writeStream(streamId, connection) 	
	for i = 1, 8 do
		streamWriteBool(streamId, self.foldAnimations[i].state); 
	end;
end;

function MurrayOctaQuad_back:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	for i = 1, 8 do
		local state = Utils.getNoNil(getXMLBool(xmlFile, key..string.format("#anim%d", i)), false);
		self:setAnimationState(i, state);
	end;
	
	return BaseMission.VEHICLE_LOAD_OK;
end;

function MurrayOctaQuad_back:getSaveAttributesAndNodes(nodeIdent)
	local attributes = "";
	for i = 1, 8 do
		attributes = attributes..string.format(' anim%d="'..tostring(self.foldAnimations[i].state)..'"', i);
	end;
	
	return attributes, nil;
end;

function MurrayOctaQuad_back:update(dt)
	if self:getIsActiveForInput() then
		if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA2) then
			if not self.gui.baseElement.visibility then
				self:open();
			else
				self:close();
			end;
		end;	
	end;
end;

function MurrayOctaQuad_back:updateTick(dt)
	-- set action texts
	
	--front stuff
	if self.octaControl.globalTogether.mouseOver and self.globalTogether then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_together_all_dis"));
	elseif self.octaControl.globalTogether.mouseOver and not self.globalTogether then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_together_all_en"));
	end;	
	
	if self.octaControl.backTogether.mouseOver and self.backTogether then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_together_front_dis"));
	elseif self.octaControl.backTogether.mouseOver and not self.backTogether then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_together_front_en"));
	end;	

	if self.octaControl.frontTogether.mouseOver and self.frontTogether then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_together_back_dis"));
	elseif self.octaControl.frontTogether.mouseOver and not self.frontTogether then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_together_back_en"));
	end;		
	
	
	if self.octaControl.foldRightFront.mouseOver and not self.globalTogether and not self.frontTogether and not self.foldAnimations[6].state then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldUp_BackLeftSafety"));
	elseif self.octaControl.foldRightFront.mouseOver and not self.globalTogether and not self.frontTogether and self.foldAnimations[6].state then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldOut_BackLeftSafety"));
	end;

	if self.octaControl.foldLeftFront.mouseOver and not self.globalTogether and not self.frontTogether and not self.foldAnimations[5].state then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldUp_BackRightSafety"));
	elseif self.octaControl.foldLeftFront.mouseOver and not self.globalTogether and not self.frontTogether and self.foldAnimations[5].state then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldOut_BackRightSafety"));
	end;	
	
	if (self.octaControl.foldLeftFront.mouseOver or self.octaControl.foldRightFront.mouseOver) and self.frontTogether and not (self.foldAnimations[5].state and self.foldAnimations[6].state) then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldUp_BackSafety"));
	elseif (self.octaControl.foldLeftFront.mouseOver or self.octaControl.foldRightFront.mouseOver) and self.frontTogether and (self.foldAnimations[5].state and self.foldAnimations[6].state) then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldOut_BackSafety"));
	end;	
	
	if self.octaControl.rightFront.mouseOver and not self.globalTogether and not self.frontTogether and not self.foldAnimations[2].state then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldUp_BackLeftArm"));
	elseif self.octaControl.rightFront.mouseOver and not self.globalTogether and not self.frontTogether and self.foldAnimations[2].state then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldOut_BackLeftArm"));
	end;
	
	if self.octaControl.leftFront.mouseOver and not self.globalTogether and not self.frontTogether and not self.foldAnimations[1].state then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldUp_BackRightArm"));
	elseif self.octaControl.leftFront.mouseOver and not self.globalTogether and not self.frontTogether and self.foldAnimations[1].state then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldOut_BackRightArm"));
	end;
	
	if (self.octaControl.leftFront.mouseOver or self.octaControl.rightFront.mouseOver) and self.frontTogether and not (self.foldAnimations[1].state and self.foldAnimations[2].state) then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldUp_BackArm"));
	elseif (self.octaControl.leftFront.mouseOver or self.octaControl.rightFront.mouseOver) and self.frontTogether and (self.foldAnimations[1].state and self.foldAnimations[2].state) then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldOut_BackArm"));
	end;
	
	
	--back stuff
	
	if self.octaControl.foldRightBack.mouseOver and not self.globalTogether and not self.backTogether and not self.foldAnimations[8].state then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldUp_FrontLeftSafety"));
	elseif self.octaControl.foldRightBack.mouseOver and not self.globalTogether and not self.backTogether and self.foldAnimations[8].state then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldOut_FrontLeftSafety"));
	end; 

	if self.octaControl.foldLeftBack.mouseOver and not self.globalTogether and not self.backTogether and not self.foldAnimations[7].state then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldUp_FrontRightSafety"));
	elseif self.octaControl.foldLeftBack.mouseOver and not self.globalTogether and not self.backTogether and self.foldAnimations[7].state then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldOut_FrontRightSafety"));
	end;	
	
	if (self.octaControl.foldLeftBack.mouseOver or self.octaControl.foldRightBack.mouseOver) and self.backTogether and not (self.foldAnimations[7].state and self.foldAnimations[8].state) then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldUp_FrontSafety"));
	elseif (self.octaControl.foldLeftBack.mouseOver or self.octaControl.foldRightBack.mouseOver) and self.backTogether and (self.foldAnimations[7].state and self.foldAnimations[8].state) then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldOut_FrontSafety"));
	end;	
	
	if self.octaControl.rightBack.mouseOver and not self.globalTogether and not self.backTogether and not self.foldAnimations[4].state then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldUp_FrontLeftArm"));
	elseif self.octaControl.rightBack.mouseOver and not self.globalTogether and not self.backTogether and self.foldAnimations[4].state then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldOut_FrontLeftArm"));
	end;
	
	if self.octaControl.leftBack.mouseOver and not self.globalTogether and not self.backTogether and not self.foldAnimations[3].state then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldUp_FrontRightArm"));
	elseif self.octaControl.leftBack.mouseOver and not self.globalTogether and not self.backTogether and self.foldAnimations[3].state then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldOut_FrontRightArm"));
	end;
	
	if (self.octaControl.leftBack.mouseOver or self.octaControl.rightBack.mouseOver) and self.backTogether and not (self.foldAnimations[3].state and self.foldAnimations[4].state) then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldUp_FrontArm"));
	elseif (self.octaControl.leftBack.mouseOver or self.octaControl.rightBack.mouseOver) and self.backTogether and (self.foldAnimations[3].state and self.foldAnimations[4].state) then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldOut_FrontArm"));
	end;
	
	-- all stuff
	
	if (self.octaControl.foldLeftBack.mouseOver or self.octaControl.foldRightBack.mouseOver or self.octaControl.foldLeftFront.mouseOver or self.octaControl.foldRightFront.mouseOver) and self.globalTogether and not (self.foldAnimations[5].state and self.foldAnimations[6].state and self.foldAnimations[7].state and self.foldAnimations[8].state) then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldUp_AllSafety"));
	elseif (self.octaControl.foldLeftBack.mouseOver or self.octaControl.foldRightBack.mouseOver or self.octaControl.foldLeftFront.mouseOver or self.octaControl.foldRightFront.mouseOver) and self.globalTogether and (self.foldAnimations[5].state and self.foldAnimations[6].state and self.foldAnimations[7].state and self.foldAnimations[8].state) then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldOut_AllSafety"));
	end;	
	
	if (self.octaControl.leftBack.mouseOver or self.octaControl.rightBack.mouseOver or self.octaControl.leftFront.mouseOver or self.octaControl.rightFront.mouseOver) and self.globalTogether and not (self.foldAnimations[1].state and self.foldAnimations[2].state and self.foldAnimations[3].state and self.foldAnimations[4].state) then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldUp_AllArm"));
	elseif (self.octaControl.leftBack.mouseOver or self.octaControl.rightBack.mouseOver or self.octaControl.leftFront.mouseOver or self.octaControl.rightFront.mouseOver) and self.globalTogether and (self.foldAnimations[1].state and self.foldAnimations[2].state and self.foldAnimations[3].state and self.foldAnimations[4].state) then
		self.octaControl.currentAction:setText(g_i18n:getText("OCTACONTROL_foldOut_AllArm"));
	end;
	
	if self.attacherVehicle ~= nil then
		if self.attacherVehicle.camIndex ~= nil then
			if self.gui.baseElement.visibility then
				self.attacherVehicle.cameras[self.attacherVehicle.camIndex].isRotatable = false;
			else
				self.attacherVehicle.cameras[self.attacherVehicle.camIndex].isRotatable = true;
			end;
		end;
	end;
end;

function MurrayOctaQuad_back:draw()
	if not self.gui.baseElement.visibility then
		g_currentMission:addHelpButtonText(g_i18n:getText("OCTACONTROL_OPENTEXT"), InputBinding.IMPLEMENT_EXTRA2);
	else
		--InputBinding.setShowMouseCursor(true);
		g_currentMission:addHelpButtonText(g_i18n:getText("OCTACONTROL_CLOSETEXT"), InputBinding.IMPLEMENT_EXTRA2);
	end;
end;

function MurrayOctaQuad_back:open()
	self.gui:open();
end;

function MurrayOctaQuad_back:close()
	self.gui:close();
end;

function MurrayOctaQuad_back:f_leftFront(isSecond)
	self:setAnimationState(1, not self.foldAnimations[1].state);
	if isSecond ~= true then
		if self.frontTogether then
			self:f_rightFront(true);
		end
		
		if self.globalTogether then
			self:f_rightFront(true);
			self:f_leftBack(true);
			self:f_rightBack(true);		
		end;
	end;
end;

function MurrayOctaQuad_back:f_rightFront(isSecond)
	self:setAnimationState(2, not self.foldAnimations[2].state);
	if isSecond ~= true then
		if self.frontTogether then
			self:f_leftFront(true);
		end
		
		if self.globalTogether then
			self:f_leftFront(true);
			self:f_leftBack(true);
			self:f_rightBack(true);		
		end;	
	end;
end;

function MurrayOctaQuad_back:f_leftBack(isSecond)
	self:setAnimationState(3, not self.foldAnimations[3].state);
	if isSecond ~= true then
		if self.backTogether then
			self:f_rightBack(true);
		end;

		if self.globalTogether then
			self:f_leftFront(true);
			self:f_rightFront(true);
			self:f_rightBack(true);		
		end;	
	end;
end;

function MurrayOctaQuad_back:f_rightBack(isSecond)
	self:setAnimationState(4, not self.foldAnimations[4].state);
	if isSecond ~= true then
		if self.backTogether then
			self:f_leftBack(true);
		end;	

		if self.globalTogether then
			self:f_leftFront(true);
			self:f_rightFront(true);
			self:f_leftBack(true);		
		end;	
	end;
end;

function MurrayOctaQuad_back:f_foldLeftFront(isSecond)
	self:setAnimationState(5, not self.foldAnimations[5].state);
	if isSecond ~= true then
		if self.frontTogether then
			self:f_foldRightFront(true);
		end;	
		
		if self.globalTogether then
			self:f_foldRightFront(true);
			self:f_foldLeftBack(true);
			self:f_foldRightBack(true);		
		end;
	end;
end;

function MurrayOctaQuad_back:f_foldRightFront(isSecond)
	self:setAnimationState(6, not self.foldAnimations[6].state);
	if isSecond ~= true then
		if self.frontTogether then
			self:f_foldLeftFront(true);
		end;

		if self.globalTogether then
			self:f_foldLeftFront(true);
			self:f_foldLeftBack(true);
			self:f_foldRightBack(true);		
		end;
	end;
end;

function MurrayOctaQuad_back:f_foldLeftBack(isSecond)
	self:setAnimationState(7, not self.foldAnimations[7].state);
	if isSecond ~= true then
		if self.backTogether then
			self:f_foldRightBack(true);
		end;
		
		if self.globalTogether then
			self:f_foldLeftFront(true);
			self:f_foldRightFront(true);
			self:f_foldRightBack(true);		
		end;
	end;
end;

function MurrayOctaQuad_back:f_foldRightBack(isSecond)
	self:setAnimationState(8, not self.foldAnimations[8].state);
	if isSecond ~= true then
		if self.backTogether then
			self:f_foldLeftBack(true);
		end;

		if self.globalTogether then
			self:f_foldLeftFront(true);
			self:f_foldRightFront(true);
			self:f_foldLeftBack(true);		
		end;		
	end;
end;

function MurrayOctaQuad_back:setAnimationState(i, state, noEventSend)
	self.foldAnimations[i].state = state;
	--print("state:"..tostring(state).." newstate:"..tostring(self.foldAnimations[i].state));
	if state then
		self:playAnimation(self.foldAnimations[i].name, self.foldAnimations[i].speed);
	else
		self:playAnimation(self.foldAnimations[i].name, -self.foldAnimations[i].speed);
	end;
	SetAnimationStateEvent.sendEvent(self, i, state, noEventSend);
end;

function MurrayOctaQuad_back:putGlobalTogether()
	self.globalTogether = not self.globalTogether;
	if self.globalTogether then
		self.octaControl.globalTogether.overlayId = self.closedLockOverlay;
		if self.backTogether then
			self.backTogether = false;
			self.octaControl.backTogether.overlayId = self.openLockOverlay;
		end;
		if self.frontTogether then
			self.frontTogether = false;
			self.octaControl.frontTogether.overlayId = self.openLockOverlay;
		end;
	else
		self.octaControl.globalTogether.overlayId = self.openLockOverlay;	
	end;
end;

function MurrayOctaQuad_back:putBackTogether()
	self.backTogether = not self.backTogether;
	if self.backTogether then
		self.octaControl.backTogether.overlayId = self.closedLockOverlay;
		if self.globalTogether then
			self.globalTogether = false;
			self.octaControl.globalTogether.overlayId = self.openLockOverlay;	
		end;
	else
		self.octaControl.backTogether.overlayId = self.openLockOverlay;
	end;
end;

function MurrayOctaQuad_back:putFrontTogether()
	self.frontTogether = not self.frontTogether;
	if self.frontTogether then
		self.octaControl.frontTogether.overlayId = self.closedLockOverlay;
		if self.globalTogether then
			self.globalTogether = false;
			self.octaControl.globalTogether.overlayId = self.openLockOverlay;	
		end;	
	else
		self.octaControl.frontTogether.overlayId = self.openLockOverlay;
	end;
end;

SetAnimationStateEvent = {};
SetAnimationStateEvent_mt = Class(SetAnimationStateEvent, Event);

InitEventClass(SetAnimationStateEvent, "SetAnimationStateEvent");

function SetAnimationStateEvent:emptyNew()
    local self = Event:new(SetAnimationStateEvent_mt);
    self.className="SetAnimationStateEvent";
    return self;
end;

function SetAnimationStateEvent:new(vehicle, i ,state)
    local self = SetAnimationStateEvent:emptyNew()
    self.vehicle = vehicle;
	self.i = i;
	self.state = state;
    return self;
end;

function SetAnimationStateEvent:readStream(streamId, connection)
    local id = streamReadInt32(streamId);
    self.vehicle = networkGetObject(id);

	self.i = streamReadInt16(streamId);
	self.state = streamReadBool(streamId);

	if self.vehicle ~= nil then
		self.vehicle:setAnimationState(self.i, self.state, true);
	end;
	if not connection:getIsServer() then
        g_server:broadcastEvent(SetAnimationStateEvent:new(self.vehicle, self.i, self.state), nil, connection, self.vehicle);
    end;
end;

function SetAnimationStateEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
	streamWriteInt16(streamId, self.i); 
	streamWriteBool(streamId, self.state); 
end;

function SetAnimationStateEvent.sendEvent(vehicle, i, state, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(SetAnimationStateEvent:new(vehicle, i, state), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(SetAnimationStateEvent:new(vehicle, i, state));
		end;
	end;
end;